/*
 * JBoss, Home of Professional Open Source.
 * Copyright 2010, Red Hat, Inc., and individual contributors
 * as indicated by the @author tags. See the copyright.txt file in the
 * distribution for a full listing of individual contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */
package org.jboss.as.console.client.search;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

/**
 * @author Harald Pehl
 */
public class TokenGroup {

    private final String token;
    private final List<Document> documents;

    public TokenGroup(String token) {
        this.token = token;
        this.documents = new ArrayList<>();
    }

    public void add(Document document) {
        documents.add(document);
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (!(o instanceof TokenGroup)) return false;

        TokenGroup that = (TokenGroup) o;

        if (!token.equals(that.token)) return false;

        return true;
    }

    @Override
    public int hashCode() {
        return token.hashCode();
    }

    public String getToken() {
        return token;
    }

    public Set<String> getDescriptions() {
        Set<String> descriptions = new HashSet<>();
        for (Document document : documents) {
            descriptions.add(document.getDescription());
        }
        return descriptions;
    }

    public Set<String> getKeywords() {
        Set<String> keywords = new HashSet<>();
        for (Document document : documents) {
            keywords.addAll(document.getKeywords());
        }
        return keywords;
    }
}
